﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Encrypted_Mutlicasting_Chat_System
{
    public partial class Chat_Room : Form
    {
        string Nick_Name = string.Empty;
        string MSGFrom = string.Empty;
        string JustTheMSG = string.Empty;
        string ServerKey = string.Empty;

        Multicasting multi = null;
        KMS.Service1 Crypt_KMS;
        CryptographyClass Crypt = new CryptographyClass(); 

        public void LogOff()
        {
            try
            {               
                if (multi != null)
                {
                    // Send the left Message
                    Send("has left the room");
                    // Stop The Listening Service
                    multi.StopListening_Service();
                }

                // Clear
                timer_Update.Enabled = false;
                ListBox_Received.Items.Clear();
                ListBox_Users.Items.Clear();
                Button_Send.Enabled = false;

                Login();
            }
            catch (Exception ex) { MessageBox.Show(ex.Message); }
        }
        void Login()
        {
            try
            {
            this.Show();
            Button_Send.Enabled = false;
            Login_Dialog log = new Login_Dialog();

            if (log.ShowDialog() == DialogResult.OK)
            {
                //Connecting with the KMS Service
                ShowStatusMessages("Connecting with the KMS server ...");
                Crypt_KMS = new Encrypted_Mutlicasting_Chat_System.KMS.Service1();
                
                ShowStatusMessages("Receiving the encryption key and starting the listening service...");
                
                //Get The Server Key as XML String
                ServerKey = Crypt_KMS.GetTheServerKey(log.RoomID);
                               
                // Join to the Multicast Group and Start The Listening Service
                multi = new Multicasting(log.MutlicastIP);
                multi.StartListening_Service();
                
                // Enable the Text Updating Timer
                timer_Update.Enabled = true; 
                // Enable the Text Sending Function
                Button_Send.Enabled = true;
                TextBox_Writing.Focus();
                Nick_Name = log.NickName;

                //-| If Successfully Connected Send the Joining Message
                Send("has joined the room");

            }
            }
            catch (Exception ex) { MessageBox.Show(ex.Message); }
        }
        void ShowStatusMessages(string MSG)
        {
            ListBox_Received.Items.Add(MSG);
            ListBox_Received.SelectedIndex = ListBox_Received.Items.Count - 1;
        }
        void UpdateTheUsersList()
        {
            try
            {
            // Update The Users List (Analysis The Commands)
            if (!ListBox_Users.Items.Contains(MSGFrom))
            {
                    ListBox_Users.Items.Add(MSGFrom);
             }
            if (JustTheMSG == " has left the room")
            {
                RemoveUserFromTheList(MSGFrom);
            }
            }
            catch (Exception ex) { MessageBox.Show(ex.Message); }
        }
        void Receive_And_Decrypttion()
        {
            try
            {
                //Read The Received Ecrypted Buffer
                byte[] EncryptedBuffer = multi.Get_The_Received_Bytes;

                if (EncryptedBuffer.Length != 0) // If The Encrypted Buffer is not empty
                {
                    // Decrypt The Message
                    textBox_ComingMSGS.Text = Crypt.RSADecryption(EncryptedBuffer, ServerKey);

                    // if the Last Message in the List equal the received message, so i dont need to print it again
                    if (textBox_ComingMSGS.Text != "" & ListBox_Received.Items[ListBox_Received.Items.Count - 1].ToString() != textBox_ComingMSGS.Text)
                    {
                         // Split The Message (The Message From And The Message content)
                         MSGFrom = textBox_ComingMSGS.Text.Remove(textBox_ComingMSGS.Text.IndexOf('#')).ToString();
                         JustTheMSG = textBox_ComingMSGS.Text.Substring(textBox_ComingMSGS.Text.IndexOf('#') + 1).ToString();

                          // Add The Message to the List
                          ListBox_Received.Items.Add(textBox_ComingMSGS.Text);
                            
                          // Select The Last Message
                          ListBox_Received.SelectedIndex = ListBox_Received.Items.Count - 1;
                          UpdateTheUsersList(); // check if the sender is not exist in my users list
                    }
                }
            }
            catch (Exception) {}
        }
        void RemoveUserFromTheList(string NickName)
        {
          ListBox_Users.Items.Remove(NickName);
        }

        void Send(string PlainMSG)
        {
            try
            {
            // forming the message (Just allow for 40 Characters to be sending) 
            //  --> Encryption --> Send the message to the multicast Group 
            string InstantText = Nick_Name + "# " + PlainMSG;
            if (multi != null)
            {
                if (InstantText.Length > 40)
                {
                    InstantText = InstantText.Substring(0, 40);
                }
                byte[] EncryptedBuffer = Crypt.RSAEncryption(ServerKey, InstantText);
                multi.Send_The_Encrypted_MSG(EncryptedBuffer);
                TextBox_Writing.Text = string.Empty;
            }
            }
            catch (Exception ex) { MessageBox.Show(ex.Message); }
        }

        public Chat_Room()
        {
            InitializeComponent();

            Login();

        }
        private void Button_Send_Click(object sender, EventArgs e)
        {
            Send(TextBox_Writing.Text);
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
        private void signOutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LogOff();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About ab = new About();
            ab.ShowDialog();
        }

        private void TextBox_Writing_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter & Button_Send.Enabled) { Send(TextBox_Writing.Text);}
        }

        private void timer_Update_Tick(object sender, EventArgs e)
        {
            Receive_And_Decrypttion();
        }
    }
}
